from enum import Enum, auto


class ConnectionState(Enum):
    """
    Represents the various states of a WebSocket connection.

    Attributes:
        DISCONNECTED: The connection is currently not active.
        CONNECTING: The connection is in the process of being established.
        CONNECTED: The connection is successfully established and active.
        CLOSED: The connection has been explicitly closed.
    """

    DISCONNECTED = auto()
    CONNECTING = auto()
    CONNECTED = auto()
    CLOSED = auto()


class ServerState(Enum):
    """
    Represents the various states of the WebSocket server itself.

    Attributes:
        CLOSED: The server is not running and not listening for connections.
        SERVING: The server is actively running and accepting new connections.
    """

    CLOSED = auto()
    SERVING = auto()


class PacketSource(Enum):
    """
    Represents the source of a packet that is being sent.

    Attributes:
        BROADCAST: A packet broadcast to all connected clients.
        CHANNEL: A packet published to a specific subscribed channel of the client.
        CUSTOM: A packet manually sent by the server.
    """

    BROADCAST = auto()
    CHANNEL = auto()
    CUSTOM = auto()
